(*******************************************************************************
tBmTbEditor

Form used for user to configure toolbar

********************************************************************************
Author name=BOURMAD Mehdi
Author E-mail=bourmad@mygale.org
Author URL=www.mygale.org/~bourmad
*******************************************************************************)
unit BmTbForm;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, ExtCtrls, StdCtrls,
  BmTbMisc, BmBmp;

type
  TBmTbEditor = class(TForm)
    PanelTrash: TPanel;
    BtnOK: TBitBtn;
    Timer1: TTimer;
    ListBoxBtn: TListBox;
    procedure MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure PanelTrashDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure PanelTrashDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure BtnOKClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListBoxBtnDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ListBoxBtnMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
    ListDelete : tList;
    procedure GetMinMaxInfo (var message : TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    { Public declarations }
    procedure AddButton2List (aCaption :String; aBitmap16, aBitmap32 : tBitmap; OnClick : tNotifyEvent);
    procedure AddBtnRes2List (aCaption, Res16, Res32 :String; OnClick : tNotifyEvent);
  end;

var
  BmTbEditor: TBmTbEditor;

implementation

{$R *.DFM}


procedure TBmTbEditor.GetMinMaxInfo (var message : TWMGetMinMaxInfo);
begin
  inherited;
  message.MinMaxInfo^.ptMaxPosition.X := Left;
  message.MinMaxInfo^.ptMaxPosition.Y := Top;
  message.MinMaxInfo^.ptMaxSize.X := Width;
  message.MinMaxInfo^.ptMaxSize.Y := Height;
  message.MinMaxInfo^.ptMinTrackSize.X := Width;
  message.MinMaxInfo^.ptMinTrackSize.Y := Height;
  message.MinMaxInfo^.ptMaxTrackSize.X := Width;
  message.MinMaxInfo^.ptMaxTrackSize.Y := Height;
end;

{IMPORTANT - Give this method to EVERY TOOLBAR BUTTON's OnMouseDown event!}
procedure TBmTbEditor.MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  with sender as TControl do
  begin
    BmTB_SendFromPool := true; {button from Editor, don't let user "Delete" it!}
    BmTB_Working := false;     {not moving existing (on toolbar) button, either}
    BeginDrag(false);
  end;
end;


procedure TBmTbEditor.FormCreate(Sender: TObject);
begin
  BorderIcons := BorderIcons - [biMaximize];
  ListDelete := tList.Create;
  PanelTrash.Caption := IdsTrashDrop;
  ListBoxBtn.Clear;
end;

procedure TBmTbEditor.FormDestroy(Sender: TObject);
begin
  ListDelete.Free;
end;

procedure TBmTbEditor.FormShow(Sender: TObject);
begin
  BmTB_SendFromPool := true; {set switch to "button from editor" mode}
  BmTB_ConfigTB := true;     {we're editing now, so disable all button methods}
end;

procedure TBmTbEditor.FormHide(Sender: TObject);
begin
  BmTB_ConfigTB := False;  {done editing, so make all button methods active}
  BmTB_Working := False;   {not dragging an existing button, either}
  if ListDelete.Count<>0
    then Timer1Timer (Sender); {remove buttons if we have ones}
end;

procedure TBmTbEditor.BtnOKClick(Sender: TObject);
begin
  close;
end;


{Use a TPanel as our "garbage can" or "black hole" to delete buttons off
toolbar.  You can use just about any component, as long as it has a
OnDragDrop event that you can access.}
procedure TBmTbEditor.PanelTrashDragDrop(Sender, Source: TObject; X, Y: Integer);
begin
  BmTB_SendFromPool := true; {keep the default "button from Editor" mode}
  BmTB_Working := false;     {reset this to "New button" mode}
  (Source as tControl).Parent.RemoveControl (Source as tControl);
  ListDelete.Add (Source);   {make the button "disappear" from toolbar}
end;

{This method will only allow buttons dragged from the Toolbar get dropped on
our TPanel for deletion. This prevents the rocket scientists of the world from
"deleting" buttons from the Editor.}
procedure TBmTbEditor.PanelTrashDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  if BmTB_SendFromPool = false
    then accept := true
    else accept := false;
end;


{This was the tricky part of this whole process.  We needed a way for a method
that was outside the tButton's realm to actually delete an invisible button from
the toolbar.  Everything that we tried using an event that in any way was
associated with the TButton in question generated an instant and profound GP
Fault.  With this thing, every 1/2 second (settable by you to a different time
frame, but we recommend a very short time frame) the timer checks to see if the
toolbar has changed (which is set by the TPanels OnDragDrop event).  If the
toolbar has changed (meaning a button has been made invisible), within 1/2
second it deletes all invisible buttons, thereby allowing a rocket scientist
user to put the same toolbar button back on the toolbar that he/she just
deleted, if he/she wants.  It works rather well.}
procedure TBmTbEditor.Timer1Timer(Sender: TObject);
var
  aControl : tControl;
begin
  while ListDelete.Count<>0 do
  begin
    aControl := ListDelete.Items[0];
    ListDelete.Remove(aControl);
    ListDelete.Pack;
    aControl.Free;
  end;
end;

procedure TBmTbEditor.AddButton2List (aCaption :String;
                                      aBitmap16, aBitmap32 : tBitmap;
                                      OnClick : tNotifyEvent);
var
  aBtnInfo : tButtonInfo;
begin
  aBtnInfo := tButtonInfo.Create (Self);
  aBtnInfo.Caption := aCaption;
  aBtnInfo.Bitmap16 := aBitmap16;
  aBtnInfo.Bitmap32 := aBitmap32;
  aBtnInfo.OnClick := OnClick;
  ListBoxBtn.Items.AddObject(aBtnInfo.Caption, aBtnInfo);
end;

procedure TBmTbEditor.AddBtnRes2List (aCaption, Res16, Res32 :String; OnClick : tNotifyEvent);
var
  aBtnInfo : tButtonInfo;
  Bmp16,
  Bmp32    : tBitmap;
  Buf : array[0..255] of Char;
begin
  aBtnInfo := tButtonInfo.Create (Self);
  if Length(Res16)<>0
  then begin
    Bmp16 := tBitmap.Create;
    Bmp16.Handle := LoadBitmap(HInstance, StrPCopy (Buf, Res16));
  end
  else begin
    Bmp16 := nil;
  end;
  if Length(Res32)<>0
  then begin
    Bmp32 := tBitmap.Create;
    Bmp32.Handle := LoadBitmap(HInstance, StrPCopy (Buf, Res32));
  end
  else begin
    Bmp32 := nil;
  end;
  aBtnInfo.Caption := aCaption;
  aBtnInfo.Bitmap16 := Bmp16;
  aBtnInfo.Bitmap32 := Bmp32;
  aBtnInfo.OnClick := OnClick;
  ListBoxBtn.Items.AddObject(aBtnInfo.Caption, aBtnInfo);
end;

procedure TBmTbEditor.ListBoxBtnDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Bitmap : TBitmap;
  Offset : Integer;
begin
  with (Control as TListBox).Canvas do
  begin
    FillRect(Rect);
    Offset := 2;
    Bitmap := TButtonInfo((Control as TListBox).Items.Objects[Index]).Bitmap32;

    if Bitmap <> nil then
    begin
      DrawTransparentBitmap ((Control as TListBox).Canvas, Bitmap,
                             Bounds(Rect.Left + 2, Rect.Top, Bitmap.Width, Bitmap.Height),
                             Bitmap.Canvas.Pixels[0,0], false);
      Offset := Bitmap.width + 6;
    end;
    TextOut(Rect.Left + Offset, Rect.Top, (Control as TListBox).Items[Index])
  end;
end;

procedure TBmTbEditor.ListBoxBtnMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
var
  BtnInfo : tButtonInfo;
  Bitmap : TBitmap;
begin
  BtnInfo := TButtonInfo((Control as TListBox).Items.Objects[Index]);
  if BtnInfo <> nil
  then begin
    Bitmap := TButtonInfo((Control as TListBox).Items.Objects[Index]).Bitmap32;
    if Bitmap <> nil then
    begin
      Height := Bitmap.Height;
    end
    else begin
      Height := 16;
    end;
  end
  else
    Height := 32;
end;

end.
